"""Write unit tests for the ``jwql`` app.

** CURRENTLY NOT IN USE **

Use pytest to run unit tests on portions of the code.

References
----------
For more information please see:
    https://docs.djangoproject.com/en/2.0/internals/contributing/writing-code/unit-tests/

"""

from django.test import TestCase

# Create your tests here.
